/***
 * OW2 FraSCAti Assembly Factory
 * Copyright (C) 2008-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org 
 *
 * Author: Damien Fournier
 * Contributor : Valerio Schiavoni
 */

package org.ow2.frascati.factory.core.instance.binding;

import java.util.HashMap;

import org.eclipse.stp.sca.BaseReference;
import org.eclipse.stp.sca.BaseService;
import org.eclipse.stp.sca.Binding;
import org.eclipse.stp.sca.ScaPackage;
import org.eclipse.stp.sca.WebServiceBinding;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.bf.BindingFactory;
import org.objectweb.fractal.bf.BindingFactoryException;
import org.objectweb.fractal.fraclet.annotations.Requires;
import org.objectweb.fractal.util.Fractal;
import org.ow2.frascati.factory.core.instance.runtime.RuntimeFactory;

/**
 * Bind components using Web Service Binding
 * 
 * @author <a href="mailto:damien.fournier@inria.fr">Damien Fournier</a>
 * @version $Revision$
 */

@org.objectweb.fractal.fraclet.annotations.Component
public class ScaBindingWs implements ScaBindingProtocol {

  /**
   * The binding type according to the SCA model
   */

  public static int BINDING_TYPE = ScaPackage.WEB_SERVICE_BINDING;

  /**
   * Logger for this class
   */

  java.util.logging.Logger log = java.util.logging.Logger
      .getLogger(ScaBindingWs.class.getCanonicalName());

  /**
   * The required binding factory
   */

  @Requires(name = "binding-factory")
  BindingFactory bf;

  /**
   * The required runtime
   */

  @Requires(name = "runtime")
  private RuntimeFactory runtime;

  /*
   * (non-Javadoc)
   * 
   * @see org.ow2.frascati.factory.core.instance.binding.ScaBindingProtocol#bind
   * (org.eclipse.stp.sca.BaseReference, org.eclipse.stp.sca.Binding,
   * org.objectweb.fractal.api.Component)
   */

  public void bind(BaseReference reference, Binding binding, Component component) {

    WebServiceBinding b = (WebServiceBinding) binding;

    String referenceName = ScaBindingUtil.getReferenceName(reference);
    
//    FeatureMap f = b.getAnyAttribute();
//    for (Entry entry : f) {
//      if(entry.getEStructuralFeature().getName().equals("option")) {
//        entry.getValue();
//      }
//    }
    
    HashMap<String, Object> m = new HashMap<String, Object>();

    // set used binding protocol
    m.put(ScaBindingProtocol.PLUGIN_ID, "ws");
    // retrieve classloader used by runtime component
    m.put(ScaBindingProtocol.CLASSLOADER, runtime.getClassLoader());
    // set protocol specific parameter
    m.put(org.objectweb.fractal.bf.connectors.ws.WsConnectorConstants.URI,
        b.getUri());
    m.put(org.objectweb.fractal.bf.connectors.ws.WsConnectorConstants.WSDL_ELEMENT, b.getWsdlElement());
    m.put(org.objectweb.fractal.bf.connectors.ws.WsConnectorConstants.WSDL_LOCATION, b.getWsdlLocation());

    try {
      log.fine("Calling binding factory\n bind : "
          + Fractal.getNameController(component).getFcName() + " -> "
          + referenceName + "\n with "
          + org.objectweb.fractal.bf.connectors.ws.WsConnectorConstants.URI
          + "=" + b.getUri()+","
          + org.objectweb.fractal.bf.connectors.ws.WsConnectorConstants.WSDL_ELEMENT
          +"="+b.getWsdlElement()
          +","+org.objectweb.fractal.bf.connectors.ws.WsConnectorConstants.WSDL_LOCATION
          + "="+ b.getWsdlLocation());

      bf.bind(component, referenceName, m);

    } catch (BindingFactoryException e) {
      e.printStackTrace();
    } catch (NoSuchInterfaceException e) {
      log.severe("Error while getting component name controller");
      e.printStackTrace();
    }

  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * org.ow2.frascati.factory.core.instance.binding.ScaBindingProtocol#export
   * (org.eclipse.stp.sca.BaseService, org.eclipse.stp.sca.Binding,
   * org.objectweb.fractal.api.Component)
   */

  public void export(BaseService service, Binding binding, Component component) {

    WebServiceBinding b = (WebServiceBinding) binding;
    HashMap<String, Object> m = new HashMap<String, Object>();

    String serviceName = ScaBindingUtil.getServiceName(service);

    // set used binding protocol
    m.put(ScaBindingProtocol.PLUGIN_ID, "ws");
    // retrieve classloader used by runtime component
    m.put(ScaBindingProtocol.CLASSLOADER, runtime.getClassLoader());
    // set protocol specific parameter
    m.put(org.objectweb.fractal.bf.connectors.ws.WsConnectorConstants.URI,
        b.getUri());
    m.put(org.objectweb.fractal.bf.connectors.ws.WsConnectorConstants.WSDL_ELEMENT, b.getWsdlElement());
    m.put(org.objectweb.fractal.bf.connectors.ws.WsConnectorConstants.WSDL_LOCATION, b.getWsdlLocation());

    try {

      log.fine("Calling binding factory\n export : "
          + Fractal.getNameController(component).getFcName() + " -> "
          + serviceName + "\n with "
          + org.objectweb.fractal.bf.connectors.ws.WsConnectorConstants.URI
          + "=" + b.getUri()+","
          + org.objectweb.fractal.bf.connectors.ws.WsConnectorConstants.WSDL_ELEMENT
          +"="+b.getWsdlElement()
          +","+org.objectweb.fractal.bf.connectors.ws.WsConnectorConstants.WSDL_LOCATION);

      bf.export(component, serviceName, m);

    } catch (BindingFactoryException e) {
      e.printStackTrace();
    } catch (NoSuchInterfaceException e) {
      log.severe("Error while getting component name controller");
      e.printStackTrace();
    }
  }

  /*
   * (non-Javadoc)
   * 
   * @seeorg.ow2.frascati.factory.core.instance.binding.ScaBindingProtocol#
   * getBindingID()
   */

  public int getBindingID() {
    return BINDING_TYPE;
  }

}
