/***
 * OW2 FraSCAti Assembly Factory
 * Copyright (C) 2008-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org 
 *
 * Author: Damien Fournier
 * 
 * Contributor(s): Nicolas Dolet
 */

package org.ow2.frascati.factory.core.instance.intent;

import java.util.List;

import javax.xml.namespace.QName;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.stp.sca.ScaPackage;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.util.Fractal;
import org.ow2.frascati.tinfi.TinfiDomain;
import org.ow2.frascati.tinfi.control.intent.IntentHandler;
import org.ow2.frascati.tinfi.control.intent.SCABasicIntentController;

/**
 * {@link ScaIntent} implementation that set intents for SCA components
 * 
 * @author <a href="mailto:nicolas.dolet@inria.fr">Nicolas Dolet</a>
 * @version $Revision$
 */
@org.objectweb.fractal.fraclet.annotations.Component
public class ComponentIntent
     extends AbstractIntentPlugin {

  // --------------------------------------------------------------------------
  // Implementation of the ScaIntent interface
  // --------------------------------------------------------------------------

  /**
   * @see org.ow2.frascati.factory.core.instance.intent.ScaIntent#getEObjectID()
   */
  public int getEObjectID() {
    return ScaPackage.COMPONENT;
  }

  /**
   * @see org.ow2.frascati.factory.core.instance.intent.ScaIntent#setIntent(Object, EObject)
   */
  public void setIntent(Object o, EObject modelElement) throws IntentException {

    // Retrieve fractal component
    Component component = (Component) o;

    // Retrieve sca (model) component
    org.eclipse.stp.sca.Component componentElement = (org.eclipse.stp.sca.Component) modelElement;

    // Get intents required for this SCA component
    List<QName> intents = componentElement.getRequires();

    for (QName name : intents) {

      // Get Intent name;
      String intentName = name.getLocalPart();

      // Try to retrieve intent component from cache
      Component intentComponent = cache.getFromCache(intentName);

      // Set business component intent
      try {
        // Stop intent component
        Fractal.getLifeCycleController(intentComponent).stopFc();

        // Get intent handler
        IntentHandler h = TinfiDomain.getService(intentComponent,
            IntentHandler.class, "intent");

        // Get intent controller
        SCABasicIntentController ic = (SCABasicIntentController) component
            .getFcInterface(SCABasicIntentController.NAME);

        // Add handler
        ic.addFcIntentHandler(h);

        // Start intent component
        Fractal.getLifeCycleController(intentComponent).startFc();

      } catch (Exception e) {
        throw new IntentException("Intent : " + intentName
            + " can not be added to component " + componentElement.getName(), e);
      }

    }

  }
}
