/***
 * OW2 FraSCAti Assembly Factory
 * Copyright (C) 2008-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org 
 *
 * Author: Damien Fournier
 */

package org.ow2.frascati.factory.core.instance.runtime;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.Type;
import org.objectweb.fractal.api.factory.GenericFactory;
import org.objectweb.fractal.api.factory.InstantiationException;
import org.objectweb.fractal.api.type.ComponentType;
import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.api.type.TypeFactory;
import org.objectweb.fractal.util.Fractal;
import org.ow2.frascati.factory.core.instance.util.AssemblyFactoryProvider;


/**
 * Runtime component acting as proxy between OW2 FraSCAti Assembly Factory and
 * Tinfi/Juliac
 * 
 * @author <a href="mailto:damien.fournier@inria.fr">Damien Fournier</a>
 * @version $Revision$
 */

@org.objectweb.fractal.fraclet.annotations.Component
public class TinfiRuntime extends Loader implements RuntimeFactory {

  /** julia.config property value */
  protected static final String JULIAC_CONFIG_FILE = "tinfi-bundled.cfg";

  /**
   * Logger for this class
   */

  java.util.logging.Logger log = java.util.logging.Logger
      .getLogger(TinfiRuntime.class.getCanonicalName());

  /** Reference for Tinfi Component */
  protected Component tinfi;

  /** Reference for Generic Factory interface */
  protected GenericFactory gf;

  /** Reference for Type Factory interface */
  protected TypeFactory tf;

  public TinfiRuntime() {

    super();

    // Ensure julia.config set to tinfi.cfg
    System.setProperty("julia.config", JULIAC_CONFIG_FILE);
    log.fine("julia.config is : " + System.getProperty("julia.config"));

    try {

      // Create providers instances
      tinfi = new AssemblyFactoryProvider().newFcInstance();
      
      // Get Generic Factory which allow to create component instances
      this.gf = Fractal.getGenericFactory(tinfi);
      
      // Get Type Factory which allow to create component types
      this.tf = Fractal.getTypeFactory(tinfi);

    } catch (InstantiationException e) {
      e.printStackTrace();
    } catch (NoSuchInterfaceException e) {
      e.printStackTrace();
    }
  }

  /**
   * @see org.objectweb.fractal.api.type.TypeFactory#createFcItfType(java.lang.String,
   *      java.lang.String, boolean, boolean, boolean)
   */

  public InterfaceType createFcItfType(String arg0, String arg1, boolean arg2,
      boolean arg3, boolean arg4) throws InstantiationException {
    return tf.createFcItfType(arg0, arg1, arg2, arg3, arg4);
  }

  /**
   * @see org.objectweb.fractal.api.type.TypeFactory#createFcType(org.objectweb.fractal.api.type.InterfaceType[])
   */

  public ComponentType createFcType(InterfaceType[] arg0)
      throws InstantiationException {
    return tf.createFcType(arg0);
  }

  /**
   * @see org.objectweb.fractal.api.factory.GenericFactory#newFcInstance(org.objectweb
   *      .fractal.api.Type, java.lang.Object, java.lang.Object)
   */

  public org.objectweb.fractal.api.Component newFcInstance(Type arg0,
      Object arg1, Object arg2) throws InstantiationException {
    try {
      Component c = gf.newFcInstance(arg0, arg1, arg2);
      return c;
    } catch (InstantiationException e) {
      // Debug used class loader on InstantiationException
      debug(null);
      throw (e);
    }
  }

}
