/***
 * OW2 FraSCAti Assembly Factory
 * Copyright (C) 2008-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org 
 *
 * Author: Damien Fournier
 */

package org.ow2.frascati.factory.core.instance.util;

import java.util.HashMap;
import java.util.Map;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.factory.InstantiationException;
import org.objectweb.fractal.util.ChainedProvider;
import org.objectweb.fractal.util.Fractal;

/**
 * Fractal provider used by AssemblyFactory
 * 
 * @author <a href="mailto:damien.fournier@inria.fr">Damien Fournier</a>
 * @version $Revision$
 */

public class AssemblyFactoryProvider extends ChainedProvider {

  // Provider Component used by Tinfi
  protected static final String TINFI_PROVIDER = "org.objectweb.fractal.juliac.runtime.Juliac";
  
  // Provider Component used by Julia
  protected static final String JULIA_PROVIDER = "org.objectweb.fractal.julia.Julia";
  
  // Provider Component used for OSGI Support
  // TODO add to providers array to use
  protected static final String OSGI_PROVIDER = "org.objectweb.fractal.juliac.felix.JuliacOSGiFelix";

  // Provider list
  protected static final String providers[] = { TINFI_PROVIDER, JULIA_PROVIDER};
  
  @SuppressWarnings("unchecked")
  @Override
  protected Component[] getBootComps() throws InstantiationException{
    int i = 0;
    Map hints;
    Map systemPropertiesMap = new HashMap(System.getProperties());
    Component[] comp = new Component[providers.length];

    for (String s : providers) {
      hints = new HashMap();
      hints.putAll(systemPropertiesMap);
      hints.put("fractal.provider", s);
      comp[i] = Fractal.getBootstrapComponent(hints);
      i++;
    }

    return comp;

  }

}
