/***
 * OW2 FraSCAti explorer
 * Copyright (C) 2008-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org
 *
 * Author: Christophe Demarey
 *
 * Contributor(s):
 * 
 */

package org.ow2.frascati.explorer.scaproperties;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.DefaultCellEditor;
import javax.swing.JOptionPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.SwingConstants;

/**
 * A custom <code>CellEditor</code> for property values other than
 * boolean (integer, string etc.).
 * 
 * @author Christophe Demarey
 * @author Jakub Kornas
 */
public class PropertyValueDefaultCellEditor
        extends DefaultCellEditor {
    private static final long serialVersionUID = 3301074317862174135L;

    private boolean initialized = false;
    private PropertyValueTextFieldActionListener actionListener = null;

    JTextField propertyEditorDefaultValue = null;
    Object cellEditorValue = null;
    int row;
    Object oldValue;
    boolean actionPerformed = false;

    /**
     * @see DefaultCellEditor#DefaultCellEditor(JTextField)
     */
    public PropertyValueDefaultCellEditor(JTextField textField) {
        super(textField);

        this.propertyEditorDefaultValue = textField;
        this.propertyEditorDefaultValue
                .setHorizontalAlignment(SwingConstants.RIGHT);
        super.setClickCountToStart(1);
    }

    /**
     * @see DefaultCellEditor#getTableCellEditorComponent(JTable, Object,
     *      boolean, int, int)
     */
    public Component getTableCellEditorComponent(final JTable table,
            final Object value, boolean isSelected, final int row1,
            final int column) {
        actionPerformed = false;
        if (!initialized) {
            this.propertyEditorDefaultValue.setFont(table.getFont());
            this.actionListener = new PropertyValueTextFieldActionListener(
                    table);
            this.propertyEditorDefaultValue.addActionListener(actionListener);

            initialized = true;
        }

        this.row = row1;
        this.oldValue = value;

        if (value != null) {
            this.propertyEditorDefaultValue.setText(value.toString());
        }

        return propertyEditorDefaultValue;
    }

    /**
     * @see DefaultCellEditor#getCellEditorValue()
     */
    public Object getCellEditorValue() {
        if (actionPerformed)
            return cellEditorValue;
        return oldValue;
    }

    private class PropertyValueTextFieldActionListener
            implements ActionListener {
        private JTable table = null;

        /**
         * @param table
         */
        public PropertyValueTextFieldActionListener(JTable table) {
            this.table = table;
        }

        public void actionPerformed(ActionEvent evt) {
            actionPerformed = true;
            String propertyName = table.getValueAt(row, 0).toString();

            String newValueString = propertyEditorDefaultValue.getText()
                    .toLowerCase().trim();
            Object newValue = null;

            boolean doUpdate = false;
            if (oldValue instanceof String) {
                newValue = propertyEditorDefaultValue.getText();
                doUpdate = true;
            } else if (oldValue instanceof Integer) {
                doUpdate = true;
                try {
                    newValue = Integer.valueOf(newValueString);
                } catch (Exception e) {
                    doUpdate = false;
                    showPropertyValueErrorMessage(propertyName, "Integer");
                }
            } else if (oldValue instanceof Short) {
                doUpdate = true;
                try {
                    newValue = Short.valueOf(newValueString);
                } catch (Exception e) {
                    doUpdate = false;
                    showPropertyValueErrorMessage(propertyName, "Short");
                }
            } else if (oldValue instanceof Long) {
                doUpdate = true;
                try {
                    newValue = Long.valueOf(newValueString);
                } catch (Exception e) {
                    doUpdate = false;
                    showPropertyValueErrorMessage(propertyName, "Long");
                }
            } else if (oldValue instanceof Double) {
                doUpdate = true;
                try {
                    newValue = Double.valueOf(newValueString);
                } catch (Exception e) {
                    doUpdate = false;
                    showPropertyValueErrorMessage(propertyName, "Double");
                }
            } else if (oldValue instanceof Float) {
                doUpdate = true;
                try {
                    newValue = Float.valueOf(newValueString);
                } catch (Exception e) {
                    doUpdate = false;
                    showPropertyValueErrorMessage(propertyName, "Float");
                }
            }
            if (doUpdate && newValue != null) {
                cellEditorValue = newValue;
                fireEditingStopped();
            } else {
                fireEditingCanceled();
            }
        }

        private void showPropertyValueErrorMessage(String propertyName,
                String requiredType) {
            JOptionPane.showMessageDialog(table, "Value of property '"
                    + propertyName + "' must be of type "
                    + requiredType.toUpperCase(), "Wrong property value",
                    JOptionPane.ERROR_MESSAGE);
        }
    }
}
