/***
 * OW2 FraSCAti Assembly Factory
 * Copyright (C) 2008-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org 
 *
 * Author: Damien Fournier
 *
 * Contributor: Philippe Merle
 */

package org.ow2.frascati.factory;

import java.io.File;
import java.io.IOException;
import java.net.URL;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.ow2.frascati.factory.core.dispatcher.Manager;
import org.ow2.frascati.factory.core.instance.runtime.RuntimeFactory;
import org.ow2.frascati.factory.core.instance.util.ScaCompositeHelper;
import org.ow2.frascati.factory.generate.runtime.RuntimeGenerate;

/**
 * @author <a href="mailto:damien.fournier@inria.fr">Damien Fournier</a>
 */
public class Factory extends AbstractGenerateFactory {
  // Constructors

  public Factory() {
    super();
  }

  // Methods

  /**
   * Load jars and create an SCA assembly instance from composite file
   * 
   * @param composite
   *          - The composite file located into the current class path, on the
   *          local file system, or at an URL
   * @param libs
   *          - Libraries required to load this composite
   * 
   * @return the SCA composite component or null if an error occurs.
   */

  @Override
  public Component getComposite(String composite, URL[] libs) {

    //Component factory = getFactory();
    Component scaComposite = null;

    RuntimeGenerate juliac;
    RuntimeFactory runtime;
    
    log.info("Starting the SCA composite : " + composite);

    try {
      juliac = (RuntimeGenerate) getFactory().getFcInterface("juliac");
      runtime = (RuntimeFactory) getFactory().getFcInterface("runtime");
      Manager load = (Manager) getFactory().getFcInterface("load");

      // juliac.init();
      // juliac.setOutputDir(output.getCanonicalFile());

      // Use Current Thread ClassLoader by default
      // WARNING : provided ClassLoader must be an URLClassLoader

      //runtime.setClassLoader(Thread.currentThread().getContextClassLoader());

      if (libs != null) {
        
        String libpath [] = new String [libs.length];
        for (int i = 0; i < libpath.length; i++) {
          libpath [i] = libs[i].getPath();
        }
        
        load.loadJars(libs);
            
        juliac.addJavaLib(libpath);
        
      }

      // Parse SCA composite.
      scaComposite = (Component) load.loadComposite(composite);

      ScaCompositeHelper.start(scaComposite);

    } catch (NoSuchInterfaceException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }

    return scaComposite;
  }

  /**
   * Get the factory main class.
   */
  @Override
  protected String getFactoryMainClass() {
    return "org.ow2.frascati.factory.FrascatiRuntime";
  }
  
  public static void run(Factory factory, String compositeName,
      String serviceName, String methodName, URL[] classpath, String output) {
    // the SCA composite
    @SuppressWarnings("unused")
    Component scaDomain = null;

    // => Run the FraSCAti runtime
    // Set the RMI SecurityManager in order to allow dynamic class loading.
    // System.setSecurityManager( new java.rmi.RMISecurityManager() );
    ClassLoader cl = Thread.currentThread().getContextClassLoader();
    URL policyURL = cl.getResource("frascati.policy");
    System.setProperty("java.security.policy", policyURL.toString());
    factory.setOutput(output);

    // factory.launch(compositeName, "r", "run");
    if ((serviceName != null) && (methodName != null)) {
      scaDomain = factory.launch(compositeName, serviceName, methodName,
          classpath);
    } else {
      scaDomain = factory.getComposite(compositeName, classpath);
      System.gc();
      System.out.println("\nPress any key to exit");
      try {
        System.in.read();
      } catch (IOException e) {
        e.printStackTrace();
      }
    }
  }

  /* (non-Javadoc)
   * @see org.ow2.frascati.factory.AbstractGenerateFactory#setOutput(java.lang.String)
   */
  public void setOutput(String output) {
    this.output = new File(output);

    // Get reference on the "juliac" interface given by the
    // Assembly factory runtime component

    try {
      RuntimeGenerate juliac = (RuntimeGenerate) getFactory().getFcInterface(
          "juliac");
      juliac.setOutputDir(this.output);

    } catch (NoSuchInterfaceException e) {
      throw new Error("Cannot get set Juliac output directory ", e);
    }

  }
}
