/***
 * OW2 FraSCAti Tinfi
 * Copyright (C) 2007-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org
 *
 * Author: Lionel Seinturier
 */

package org.ow2.frascati.tinfi.juliac;

import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.juliac.Juliac;
import org.objectweb.fractal.juliac.desc.MembraneDesc;
import org.objectweb.fractal.juliac.proxy.InterfaceImplementationClassGenerator;
import org.objectweb.fractal.juliac.proxy.ProxyClassGenerator;
import org.ow2.frascati.tinfi.TinfiDomain;

/**
 * This class is a factory for interface source code generators.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille1.fr>
 */
public class InterfaceSourceCodeGeneratorFactory {
    
    private InterfaceType it;
    private String ctrlDesc;
    private MembraneDesc<?> membraneDesc;
    private Juliac jc;
    
    public InterfaceSourceCodeGeneratorFactory(
        InterfaceType it, String ctrlDesc, MembraneDesc<?> membraneDesc,
        Juliac jc ) {
        
        this.it = it;
        this.ctrlDesc = ctrlDesc;
        this.membraneDesc = membraneDesc;
        this.jc = jc;
    }
    
    /**
     * Return true if the interface corresponds to a business interface of a SCA
     * component.
     */
    public boolean isSCACompBusinessInterface() {
        String name = it.getFcItfName();
        boolean b =
            ctrlDesc.equals(TinfiDomain.SCA_PRIMITIVE) &&
            !( name.endsWith("-controller") || name.equals("component") );
        return b;
    }
    
    /**
     * Return the source file generator for this factory.
     */
    public ProxyClassGenerator getInterfaceSourceCodeGenerator() {
        ProxyClassGenerator scg = null;
        if( isSCACompBusinessInterface() ) {
            scg =
                it.isFcClientItf() ?
                new ClientInterfaceClassGenerator(it,membraneDesc,false,jc) :
                new InterfaceImplementationClassGenerator(it,membraneDesc,false,jc);
        }
        else {
            scg = new InterfaceImplementationClassGenerator(it,membraneDesc,false,jc);
        }
        return scg;
    }
}
