/***
 * OW2 FraSCAti Tinfi
 * Copyright (C) 2007-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org
 *
 * Author: Lionel Seinturier
 */

package org.ow2.frascati.tinfi.control.content;

import java.lang.annotation.Annotation;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

/**
 * This class manages either a {@link Field} or a {@link Method} annotated by a
 * SCA annotation (typically @{@link org.osoa.sca.annotations.Reference} or
 * @{@link org.osoa.sca.annotations.Property}).
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille1.fr>
 */
public class AnnotatedElement {
    
    private Field field = null;
    private Method method = null;
    
    public AnnotatedElement( Field field ) {
        this.field = field;
        field.setAccessible(true);
    }
    public AnnotatedElement( Method method ) {
        this.method = method;
        method.setAccessible(true);
    }
    
    public void set( Object target, Object value )
    throws IllegalAccessException, InvocationTargetException {
        
        if( field == null ) {
            method.invoke( target, new Object[]{value} );
        }
        else {
            field.set(target,value);
        }
    }
    
    public Object get( Object target )
    throws IllegalAnnotatedElementOperationException, IllegalAccessException {
        
        if( field == null ) {
            String msg = "Can not get the value of a reference which is defined with a setter method";
            throw new IllegalAnnotatedElementOperationException(msg);
        }
        else {
            Object value = field.get(target);
            return value;
        }
    }
    
    public <T extends Annotation> T getAnnotation( Class<T> annotcl ) {
        T annot =
            (field == null) ?
            method.getAnnotation(annotcl) :
            field.getAnnotation(annotcl);
        return annot;
    }
}
