/***
 * OW2 FraSCAti Tinfi
 * Copyright (C) 2008-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org
 *
 * Authors: Lionel Seinturier, Philippe Merle
 */

package org.ow2.frascati.tinfi.control.content.scope;

import java.util.HashMap;
import java.util.Map;
import java.util.Stack;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.control.NameController;
import org.objectweb.fractal.api.factory.InstantiationException;
import org.osoa.sca.RequestContext;
import org.ow2.frascati.tinfi.control.component.ReconfigurableComponentContext;
import org.ow2.frascati.tinfi.control.content.ContentClassMetaData;
import org.ow2.frascati.tinfi.control.property.SCAPropertyController;

/**
 * Manager for request-scoped components.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille1.fr>
 * @author Philippe Merle <Philippe.Merle@inria.fr>
 */
public class RequestScopeManager extends AbstractScopeManager
implements ScopeManager {

    public RequestScopeManager(
        Component comp, SCAPropertyController scapc,
        ReconfigurableComponentContext compCtx, NameController nc,
        ContentClassMetaData ccmd ) {
    
        super(comp,scapc,compCtx,nc,ccmd);
    }
    
    public Object getFcContent() throws InstantiationException {
        Stack<RequestContext> stack = tl.get();
        Object content = null;
        
        if( stack.size() == 1 ) {
            /*
             * First call.
             * Create a content instance and register it in the map.
             */
            content = createAndInitializeFcInstance();
            contentMapRequestScope.put(stack,content);
        }
        else {
            /*
             * Retrieve the content instance from the map.
             * We can not use
             *      content = contentMapRequestScope.get(stack)
             * as the hashcode of stack has changed since it has been put in
             * the map (some elements have been pushed on the stack).
             */
            for (Map.Entry<Stack<RequestContext>, Object> entry: contentMapRequestScope.entrySet()) {
                Stack<RequestContext> key = entry.getKey();
                if( key.equals(stack) ) {
                    content = entry.getValue();
                }
            }
            if( content == null ) {
                String msg = "Reentrant call: a content instance should have been found";
                throw new InstantiationException(msg);
            }
        }
        
        return content;
    }
    
    public void releaseFcContent( Object content, boolean isEndMethod ) {

        Stack<RequestContext> stack = tl.get();
        if( stack.size() == 1 ) {
            /*
             * Return of the first call.
             * The session ends.
             */
            destroyFcInstance(content);            
            contentMapRequestScope.remove(stack);
        }
    }

    /**
     * A map of content instances for request scoped components.
     * The key is the thread-local request task (from tl).
     * The value is the associated content instance.
     */
    private Map<Stack<RequestContext>,Object> contentMapRequestScope =
        new HashMap<Stack<RequestContext>,Object>();
}
