/***
 * OW2 FraSCAti Tinfi
 * Copyright (C) 2007-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org
 *
 * Author: Lionel Seinturier
 */

package org.ow2.frascati.tinfi.control.property;

import java.util.Map;

import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.julia.type.BasicInterfaceType;

/**
 * Property control interface for SCA primitive components.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille1.fr>
 */
public interface SCAPropertyController {
    
    /** <code>NAME</code> of the content controller. */
    final public static String NAME = "sca-property-controller";
    
    /** <code>TYPE</code> of the content controller. */
    final public static InterfaceType TYPE =
        new BasicInterfaceType(
            NAME,
            SCAPropertyController.class.getName(),
            false, false, false );    

    /**
     * Set the value for the specified property name. If the property has
     * already been set, the old value is lost, and the new value is recorded.
     * 
     * @param name   the property name
     * @param value  the property value
     */
    public void set( String name, Object value );
    
    /**
     * Return the value for the specified property name. Return null if the
     * property has not been set.
     * 
     * @param name  the property name
     * @return      the property value
     */
    public Object get( String name );
    
    /**
     * Return <code>true</code> if the specified property name has been set.
     * 
     * @param name  the property name
     * @return      <code>true</code> if the property has been set,
     *              <code>false</code> otherwise
     */
    public boolean containsKey( String name );
    
    /**
     * Return a map of all the properties which have been set so far.
     * 
     * @return  a map of all the properties which have been set so far
     */
    public Map<String,Object> getAll();

    /**
     * Set the reference of the property controller which promotes the specified
     * property to the current property controller.
     * 
     * @param name      the promoter property name
     * @param promoter  the promoter component or
     *                  <code>null</code> to unregister the promoter
     * @throws IllegalPromoterException
     *      thrown when attempting to set a cycle between property promoters
     * @since 0.4.3
     */
    public void setPromoter( String name, SCAPropertyController promoter )
    throws IllegalPromoterException;
    
    /**
     * Return the reference of the property controller which promotes the
     * specified property. Return <code>null</code> if the property is managed
     * locally by the current property controller.
     * 
     * @param name  the promoter property name
     * @return      the promoter component or <code>null</code>
     * @since 0.4.3
     */
    public SCAPropertyController getPromoter( String name );
}
